'use strict';

// On instancie l'application sauvonsTombes avec les différentes dépendances liées.
var sauvonsTombes = angular.module('sauvonsTombes', [
    'ngRoute',

    'pascalprecht.translate',
    'GeneanetService',

    'sauvonsTombesAnimations',
    'sauvonsTombesControllers',
    'sauvonsTombesFilters',
    'sauvonsTombesServices',
    'sauvonsTombesDirectives',
]);

// Action effectuée au chargement de l'application.
sauvonsTombes.run([
    '$rootScope',
    'GeneanetDatabase',
    'GeneanetOverlay',
    function ($rootScope, GeneanetDatabase, GeneanetOverlay) {
        // Connexion à la bdd
        GeneanetDatabase.initConnexion();
        window.addEventListener('resize',function() {$rootScope.$broadcast("resize", {});});

        document.addEventListener("backbutton", function() {
            if (GeneanetOverlay.isOpen() && GeneanetOverlay.isClosable()) {
                GeneanetOverlay.close();
            } else {
                if (
                    window.location.hash.split("/")[1] !== undefined &&
                    window.location.hash.split("/")[1] !== ""
                ) {
                    window.history.back();
                } else {
                    navigator.app.exitApp();
                }
            }
        });
    }
]);

sauvonsTombes.constant('urlsWebservice', function() {
    var baseUrl;
    if (ENV_VAR.env == "prod") {
        baseUrl = "https://api.geneanet.org/v1/";
    } else {
        baseUrl = "https://api."+ENV_VAR.envdev+".gntdev.com/v1/";
    }

    return {
        baseUrl: baseUrl,
        galleryUrl: baseUrl + "gallery/",
        connexionUrl: baseUrl + "connexion/v4/?ctype=id&beta=gntvip"
    };
}());

// Rooting de l'application
sauvonsTombes.config(['$routeProvider',
    function($routeProvider) {
        $routeProvider.
            when('/galleryProject/:inverseOrderBy?', {
                templateUrl: 'views/gallery_project.html',
                controller: 'GalleryProjectController'
            }).
            when('/pictureReader/:idPicture?/:readTotalProject?', {
                templateUrl: 'views/picture_reader.html',
                controller: 'PictureReaderController',
                orientationLock: 'portrait'
            }).
            when('/listProjectGnn/', {
                templateUrl: 'views/list_project_gnn.html',
                controller: 'ListProjectGnnController'
            }).
            when('/listProjectExist/:pageType', {
                templateUrl: 'views/list_project_exist.html',
                controller: 'listProjectExistController'
            }).
            when('/cameraChoix/', {
                templateUrl: 'views/camerapage_choix.html',
                controller: 'CameraChoixController',
                orientationLock: 'portrait'
            }).
            when('/createProject/', {
                templateUrl: 'views/create_projectpage.html',
                controller: 'CreateProjectController'
            }).
            when('/detailsProject/:idProject', {
                templateUrl: 'views/details_project.html',
                controller: 'DetailsProjectController'
            }).
            when('/configuration/', {
                templateUrl: 'views/configuration.html',
                controller: 'ConfigurationController'
            }).
            when('/configuration/help/', {
                templateUrl: 'views/help.html',
                controller: 'HelpController',
                orientationLock: 'portrait'
            }).
            when('/configuration/settings/', {
                templateUrl: 'views/settings.html',
                controller: 'SettingsController'
            }).
            when('/configuration/statistics/', {
                templateUrl: 'views/statistics.html',
                controller: 'StatisticsController'
            }).
            when('/synchro/:noAlertWifi?', {
                templateUrl: 'views/synchro.html',
                controller: 'SynchroController'
            }).
            when('/connexion/', {
                templateUrl: 'views/connexion.html',
                controller: 'ConnexionController',
                orientationLock: 'portrait'
            }).
            when('/', {
                templateUrl: 'views/homepage.html',
                controller: 'HomepageController',
                orientationLock: 'portrait'
            }).
            otherwise({
                redirectTo: '/'
            });
  }
]);

// Configuration du module de traduction
sauvonsTombes.config(function ($translateProvider) {
    $translateProvider.useStaticFilesLoader({
        prefix: 'translations/translation.',
        suffix: '.json'
    });

    $translateProvider.determinePreferredLanguage(determinePreferredLanguage);
});

/**
 * Permet de déterminer la langue à utiliser.
 *
 * @return string Retourne la langue de l'appli.
 */
function determinePreferredLanguage() {
    var list_lang = new Array('fr', 'es', 'pt', 'sv', 'de', 'fi', 'it', 'nl', 'no');
    var langAsso = {
        "nb": "no"
    }

    var current_lang = navigator.language.substr(0,2);
    if (langAsso[current_lang]) {
        current_lang = langAsso[current_lang];
    }

    if (list_lang.indexOf(current_lang) > -1) {
        return current_lang;
    } else {
        return 'en';
    }
}

// Configuration liée à la base de données
sauvonsTombes.config(
    function ( GeneanetDatabaseProvider ) {
        var tabRequest = new Array();
        tabRequest.push('CREATE TABLE IF NOT EXISTS "projet" ("id_projet_local" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAULT (null) , "id_projet_gnn" INTEGER DEFAULT (null) , "nom_projet" VARCHAR, "place" VARCHAR, "type_projet" VARCHAR, "date_creation" DATETIME NOT NULL DEFAULT DATETIME, "is_sync" BOOL NOT NULL, "geo" VARCHAR NOT NULL DEFAULT (null), "proprietaire" VARCHAR)');
        tabRequest.push('CREATE TABLE IF NOT EXISTS "depot" ("id_depot" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAULT (null), "id_projet_local" INTEGER, "lieu_depot" VARCHAR DEFAULT (null), "base64_portrait" TEXT DEFAULT(null) , "base64_landscape" TEXT DEFAULT(null), FOREIGN KEY(id_projet_local) REFERENCES projet(id_projet_local) ON DELETE CASCADE ON UPDATE CASCADE)');
        tabRequest.push('CREATE TABLE IF NOT EXISTS "photo" ("id_picture" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAULT (null), "native_url" VARCHAR NOT NULL DEFAULT (null) , "id_depot" INTEGER NOT NULL DEFAULT (null) , "is_sync" BOOL NOT NULL , FOREIGN KEY(id_depot) REFERENCES depot(id_depot) ON DELETE CASCADE ON UPDATE CASCADE)');
        tabRequest.push('CREATE TABLE IF NOT EXISTS "configuration" ("key" VARCHAR PRIMARY KEY NOT NULL, "value" TEXT DEFAULT(null))');

        GeneanetDatabaseProvider.initDataBase('GeneaGraves', '1.0.1', 'GeneaGraves', 1024 * 2048, tabRequest);
    }
);

// Configuration du service pour la Camera
sauvonsTombes.config(
    function ( GeneanetCameraProvider ) {
        //Définir le niveau de qualité de l'appareil Photo
        GeneanetCameraProvider.defaultQuality( 50 );
    }
);

// Configuration du service pour l'application
sauvonsTombes.config(
    function ( GeneanetAppProvider ){
        GeneanetAppProvider.determineName("GeneaGraves");
    }
);

// Configuration du service Device
sauvonsTombes.config(
    function ( GeneanetDeviceProvider ){
        GeneanetDeviceProvider.setNameDirectory("GeneaGraves");
    }
);

// Configuration du service pour les overlay & spinner
sauvonsTombes.config(
    function (GeneanetOverlayProvider) {
        GeneanetOverlayProvider.setBackgroundColor("black");
    }
);
