'use strict';
        
var createProjectController = [
    '$scope',
    '$http',
    '$filter',
    '$translate',
    'GenerateSqlService',
    'GeneanetDevice',
    'GeneanetDatabase',
    'GeneanetGeolocation',
    'GeneanetOverlay',
    'ProjectService',
    function($scope, $http, $filter, $translate, GenerateSqlService, GeneanetDevice, GeneanetDatabase, GeneanetGeolocation, GeneanetOverlay, ProjectService) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        var formIsSubmitted = false;
        var editProject = false;
        var currentCountry = null;

        var tabTypeProjet = [
            {name: 'create_projectpage.civil', value: 'cimetiere'},
            {name: 'create_projectpage.military', value: 'cimetiere_militaire'},
            {name: 'create_projectpage.monument_death', value: 'monument'},
            {name: 'create_projectpage.church', value: 'eglise'},
            {name: 'create_projectpage.monument_other', value: 'plaque'}
        ];

        $scope.back = function (){
            window.history.back();
        }

        var showElementFromId = function (id) {
            angular.element("#"+id).css('display','block');
        }

        var hideElementFromId = function (id) {
            angular.element("#"+id).css('display','none');
        }

        /**
         * Permet d'assigner les types de projet au template.
         */
        var populateTypeProject = function () {
            $scope.tProject = new Array();
            for (var item in tabTypeProjet){
                $scope.tProject.push({name: $translate.instant(tabTypeProjet[item].name), value: tabTypeProjet[item].value});
            }

            $scope.tProject.unshift({name: $translate.instant('create_projectpage.type_project'), value: 0});
            $scope.typeProject = $scope.tProject[0];
        }

        var updateCountry = function (geoRessources) {
            // on ajoute a chaque pays son nom dans la langue courrante.
            for (var key in geoRessources) {
                geoRessources[key].name = geoRessources[key]["name_" + $translate.preferredLanguage()];
            }

            // On trie les données
            geoRessources = $filter('orderBy')(geoRessources, "name");

            // on passe la liste des pays a la vue.
            $scope.countries = geoRessources;
        }

        $scope.blurFieldValid = function() {
            if (formIsSubmitted) {
                validForm();
            }
        }

        $scope.changeCountry = function() {
            var search = $scope.searchCountry;

            if (!search || search.trim().length < 1) {
                angular.element("#select-country ul li").each(function(){
                    angular.element(this).removeClass("hide");
                });
            } else {
                var reg = new RegExp("^"+search, "i");
                for (var i in $scope.countries) {
                    if ($scope.countries[i].name.match(reg)) {
                        angular.element("#select-country #list-"+i).removeClass("hide");
                    } else {
                        angular.element("#select-country #list-"+i).addClass("hide");
                    }
                }
            }
        }

        $scope.displayList = function(event) {
            event.stopPropagation();

            $scope.changeCountry();
            angular.element("#select-country ul").addClass("display");
        }

        $scope.hideList = function() {
            angular.element("#select-country ul").removeClass("display");

            var validElements = angular.element("#select-country li:not(.hide)");
            if (validElements.length == 1 && currentCountry == null) {
                var index = angular.element(validElements[0]).data("index");
                $scope.selectValue($scope.countries[index]);
            }

            if ($scope.searchCountry && currentCountry == null) {
                $scope.$apply(function() {$scope.searchCountry = undefined});
            } else if (currentCountry != null) {
                $scope.$apply(function() {$scope.searchCountry = currentCountry.name});
            }
        }

        angular.element("html").click(function(){
            $scope.hideList();
        });

        $scope.selectValue = function(element) {
            $scope.searchCountry = element.name;
            currentCountry = element;

            hideElementFromId("select-subregion");
            $scope.currentSubRegion = null;
            if (formIsSubmitted) {
                validForm();
            }

            if (currentCountry.regions) {
                showElementFromId("select-region");

                var regions = currentCountry.regions;
                regions = $filter('orderBy')(regions, "name");
                // on rajoute le message par défaut du select du region.
                regions.unshift({name: $translate.instant('create_projectpage.region'), value: 0});

                // on set la région courrante par celle par défaut.
                $scope.currentRegion = regions[0];
                // on passe la liste des régions a la vue.
                $scope.regions = regions;
            } else {
                hideElementFromId("select-region");
                $scope.currentRegion = null;
            }
        }

        $scope.changeRegion = function () {
            if (formIsSubmitted) {
                validForm();
            }

            if ($scope.currentRegion.subregions) {
                showElementFromId("select-subregion");

                var subregions = $scope.currentRegion.subregions;
                
                subregions = $filter('orderBy')(subregions, "name");
                
                // on rajoute le message par défaut du select du sous-region. A TRADUIRE.
                subregions.unshift({name: $translate.instant('create_projectpage.subregion'), value: 0});
                
                // on set la sous région courrante par celle par défaut.
                $scope.currentSubRegion = subregions[0];

                // on passe la liste des sous-régions a la vue.
                $scope.subRegions = subregions;
            } else {
                hideElementFromId("select-subregion");
                $scope.currentSubRegion = null;
            }
        }

        /**
         * Verifie les champs du formulaire et modifie la couleur de bordure en fonction de si c'est bon ou non.
         * 
         * @param  {boolean} openOverlayAtEnd Permet de savoir si on doit ouvrir ou non l'overlay à la fin.
         * 
         * @return {boolean} true si formulaire valide, false si non.
         */
        var validForm = function (openOverlayAtEnd){
            var tabError = new Array ();
            var nameClassError = "error";

            // on veut un nom de projet
            if (!$scope.nameProjet || $scope.nameProjet.trim() == "") {
                tabError.push($translate.instant("create_projectpage.error_submit_name_project"));
                angular.element('#name-project').addClass(nameClassError);
            } else {
                angular.element('#name-project').removeClass(nameClassError);
            }

            // on veut un type de projet
            if ($scope.typeProject.value == 0) {
                tabError.push($translate.instant("create_projectpage.error_submit_type_project"));
                angular.element('#select-type').addClass(nameClassError);
            } else {
                angular.element('#select-type').removeClass(nameClassError);
            }

            // on veut forcément une position dans le cas de la création d'un projet
            if (!editProject) {
                if (!$scope.latitude || !$scope.longitude) {
                    // pas de coordonnées, on va vérifier pays / region / ss region
                    if (currentCountry == null) {
                        // pas de pays
                        tabError.push($translate.instant("create_projectpage.error_submit_country"));
                        angular.element('#select-country input').addClass(nameClassError);
                    } else {
                        angular.element('#select-country input').removeClass(nameClassError);
                        if (currentCountry.regions && (!$scope.currentRegion || !$scope.currentRegion.symbol)) {
                            tabError.push($translate.instant("create_projectpage.error_submit_region"));
                            angular.element('#select-region').addClass(nameClassError);
                        } else if (currentCountry.regions) {
                            angular.element('#select-region').removeClass(nameClassError);
                            if ($scope.currentRegion.subregions && 
                                (!$scope.currentSubRegion || !$scope.currentSubRegion.symbol)
                            ) {
                                tabError.push($translate.instant("create_projectpage.error_submit_subregion"));
                                angular.element('#select-subregion').addClass(nameClassError);
                            } else {
                                angular.element('#select-subregion').removeClass(nameClassError);
                            }
                        }
                    }
                }
            }

            // on veut un nom de commune
            if (!$scope.city || $scope.city.trim() == "") {
                tabError.push($translate.instant("create_projectpage.error_submit_city"));
                angular.element('#name-city').addClass(nameClassError);
            } else {
                angular.element('#name-city').removeClass(nameClassError);
            }

            if (tabError.length == 0) {
                return true;    
            } else {
                if (openOverlayAtEnd != "undefined" && openOverlayAtEnd) {
                    var titleOverlay = "";
                    if (tabError.length <= 1) {
                        titleOverlay = $translate.instant("create_projectpage.error_submit_field");
                    } else {
                        titleOverlay = $translate.instant("create_projectpage.error_submit_fields");
                    }

                    var contentOverlay = "<ul style='padding-left:5px;'>";
                    for (var i in tabError) {
                        contentOverlay += "<li>" + tabError[i] + "</li>";
                    }
                    contentOverlay += "</ul>";

                    GeneanetOverlay.openOverlay(titleOverlay, contentOverlay);
                }

                return false;
            }
        }

        $scope.submit = function(){
            formIsSubmitted = true;
            if (validForm(true)) {
                if (editProject) {
                    var requestUpdateProject = GenerateSqlService.updateProject(ProjectService.getCurrentIdProject(), $scope.nameProjet, $scope.city, $scope.typeProject.value);
                    GeneanetDatabase.executeRequest(requestUpdateProject, function(){window.history.go(-1);});
                } else {
                    var elementsGeo = new Array();
                    if ($scope.longitude != null && $scope.latitude != null) {
                        elementsGeo.push($scope.latitude);
                        elementsGeo.push($scope.longitude);
                    } else {
                        elementsGeo.push(currentCountry.symbol);
                        if ($scope.currentRegion) {
                            elementsGeo.push($scope.currentRegion.symbol);
                            if ($scope.currentSubRegion) {
                                elementsGeo.push($scope.currentSubRegion.symbol);
                            }
                        }
                    }

                    var request = GenerateSqlService.insertionProjet(null, $scope.nameProjet, $scope.city, $scope.typeProject.value, elementsGeo.join(","), null);
                    GeneanetDatabase.executeRequest(request, function(){window.history.go(-2);});
                }
            }
        }

        if (ProjectService.getCurrentIdProject() != null) {
            editProject = true;
        }
        
        populateTypeProject();

        // dans le cas de la création d'un projet.
        if (!editProject) {
            $scope.titlePage = 'create_projectpage.create_project';

            // on essaye de localiser seulement si on est en création de projet.
            GeneanetOverlay.openSpinner($translate.instant('spinner.try_localise'));
            
            $scope.$on("needSelectGeo", function(event, data){
                $http.get('datas/geoRessources.json').success(
                    function (data) {
                        updateCountry(data);
                    }
                );
            });

            // action sur la tentative de localisation.
            $scope.$on("localiseUser", function(event, data){
                GeneanetOverlay.close();
                if (data.success) {
                    // open this overlay juste one time by session.
                    if (!sessionStorage.getItem("overlayAlertCreateProject")) {
                        GeneanetOverlay.openOverlay(
                            $translate.instant("alert.important"),
                            $translate.instant("alert.warning_geo_loc")
                        );
                        sessionStorage.setItem("overlayAlertCreateProject", true);
                    }

                    $scope.latitude = data.position.coords.latitude;
                    $scope.longitude = data.position.coords.longitude;
                } else {
                    $scope.$broadcast("needSelectGeo", {});
                    showElementFromId("select-country");
                }
            });
            GeneanetGeolocation.getCurrentPosition("localiseUser", 60000, 5000, true);
        } else {
            $scope.titlePage = 'create_projectpage.edit_project';

            var successGetInfoProject = function(tx, results){
                if (results.rows.length > 0) {
                    var infosProject = results.rows.item(0);
                    var indexType = 0;
                    for(var i in $scope.tProject) {
                        if ($scope.tProject[i].value === infosProject.type_projet) {
                            indexType = i;
                            break;
                        }
                    }
                    $scope.$apply(function(){
                        $scope.nameProjet = infosProject.nom_projet;
                        $scope.city = infosProject.place;
                        $scope.typeProject = $scope.tProject[indexType];
                    });
                } else {
                    console.error("Impossible de récupérer les informations sur le projet en cours.");
                }
            }

            // on load les infos du projet à modifier
            var requestGetInfoProject = GenerateSqlService.getProjectsFromIds(ProjectService.getCurrentIdProject());
            GeneanetDatabase.executeRequest(requestGetInfoProject, successGetInfoProject);
        }
    }
];