'use strict';
        
var synchroController = [
    "$scope",
    "$location",
    "$timeout",
    "$route",
    "$routeParams",
    "$translate",
    "GeneanetDevice",
    "GeneanetOverlay",
    "GeneanetDatabase",
    "GeneanetFile",
    "ConnexionService",
    "GenerateSqlService",
    "ProjectService",
    "ConfigurationService",
    "GeneanetDate",
    "GeneanetLocalStorage",
    "urlsWebservice",
    function($scope, $location, $timeout, $route, $routeParams, $translate, GeneanetDevice, GeneanetOverlay, GeneanetDatabase, GeneanetFile, ConnexionService, GenerateSqlService, ProjectService, ConfigurationService, GeneanetDate, GeneanetLocalStorage, urlsWebservice) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        $scope.noProjet = true;
        $scope.currentIdProject = ProjectService.getCurrentIdProject();
        
        var urlGnn = encodeURI(urlsWebservice.galleryUrl + "synchro/image");
        var listProject = {};
        var idsProjectsChecked = new Array();
        var deleteFiles = false;
        var synchroInProgress = false;
        var errorDuringUpload = false;

        /**
         * Object permettant de gérer les actions lourdes de la page.
         * 
         * @type {Object} managerSynchProgress
         */
        var managerSynchProgress = {
            count: 0,
            init: function(value) {
                this.count = value;
            },
            advance: function() {
                this.count--;
                if (this.count <= 0) {
                    $scope.$broadcast("synchroPicturesFinished", {});
                }
            },
            stop: function() {
                this.count = 0;
                $scope.$broadcast("synchroPicturesFinished", {});
            }
        }

        /**
         * Fonction permettant de vérifier la connexion à internet.
         */
        var checkConnexion = function() {
            // check connexion
            if (!GeneanetDevice.accessInternetStarted()) {
                $scope.$on("GeneanetOverlayClose", function() {
                    $scope.back();
                });
                
                GeneanetOverlay.openOverlay(
                    $translate.instant("alert.offline"),
                    $translate.instant("alert.cant_synchro")
                );
            } else {
                $scope.$on("checkConnexionValid", function(event, data) {
                    if (data.success) {
                        GeneanetOverlay.close();
                        if (!GeneanetDevice.wifiStarted() && !$routeParams.noAlertWifi) {
                            GeneanetOverlay.openOverlay(
                                $translate.instant("alert.not_wifi"),
                                $translate.instant("alert.wifi_better")
                            );
                        }
                        $scope.$broadcast("checkFinished", {});
                    } else {
                        redirectNoConnexion();
                    }
                });
                // check la connexion à GNN.
                GeneanetOverlay.openSpinner($translate.instant("spinner.check_connexion"));
                ConnexionService.isConnected("checkConnexionValid");
            }
        }

        /**
         * Méthode permettant de commencer la synchronisation (ne gère pas l'envoie).
         */
        $scope.startSynchro = function() {
            if (!synchroInProgress) {
                synchroInProgress = true;

                idsProjectsChecked = new Array();
                angular.element("#synchro input[type='checkbox']:checked").each(function() {
                    idsProjectsChecked.push(angular.element(this).val());
                });

                if (idsProjectsChecked.length == 0) {
                    GeneanetOverlay.openOverlay(
                        $translate.instant("alert.synchronize_project_no_selected"),
                        $translate.instant("alert.synchronize_need_project_selected")
                    );
                    synchroInProgress = false;
                } else {
                    $scope.$on("overlayDialogResponse", function(event, data) {
                        if (data.response) {
                            deleteFiles = data.response;
                        } else {
                            deleteFiles = data.response;
                        }

                        synchroProjects();
                    });

                    var queryMsg = $translate.instant("dialog.delete_files");
                    var yesMsg = $translate.instant("dialog.yes");
                    var noMsg = $translate.instant("dialog.no");

                    // demande à l'utilisateur si on veut supprimer les photos une fois synchronisées ou pas.
                    GeneanetOverlay.openDialog(queryMsg, yesMsg, "btn btn-gnn-primary", noMsg, "btn btn-gnn-secondary");
                }
            }
        }

        /**
         * Permet de synchroniser les projets séléctionnés.
         */
        var synchroProjects = function() {
            // on parcourt les projets séléctionnés.
            for(var i in idsProjectsChecked) {
                var currentProject = listProject[idsProjectsChecked[i]];
                var isEdit = false;

                // on regarde s'il s'agit d'une édition ou d'un nouveau projet.
                if (
                    currentProject.id_projet_gnn == null || 
                    (currentProject.proprietaire != null && currentProject.is_sync === "false")
                ) {
                    isEdit = true;
                }

                // on démarre une progress bar.
                GeneanetOverlay.openProgressBar(
                    $translate.instant("progress_bar.synchro_project"),
                    0,
                    idsProjectsChecked.length
                );

                var params = {};
                if (currentProject.id_projet_gnn != null) {
                    params["id"] = currentProject.id_projet_gnn;
                }
                if (isEdit) {
                    params["place"] = currentProject.place;
                    params["nom_projet"] = currentProject.nom_projet;
                    params["actif"] = 0;
                    params["type_projet"] = currentProject.type_projet;

                    var geoCurrent = currentProject.geo.split(",");
                    if (geoCurrent.length == 2 && !isNaN(geoCurrent[0]) && !isNaN(geoCurrent[1])) {
                        params["lat"] = geoCurrent[0];
                        params["lon"] = geoCurrent[1];
                    } else {
                        params["country"] = geoCurrent[0];
                        if (geoCurrent.length > 1) {
                            params["region"] = geoCurrent[1];
                            if (geoCurrent.length > 2) {
                                params["subregion"] = geoCurrent[2];
                            }
                        }
                    }
                }

                // fonction permettant d'envoyer le projet à GNN.
                (function(currentProject) {
                    $.post(urlsWebservice.galleryUrl + "projects/add", params)
                        .success(function(data) {
                            // si nn connecté, on redirige à la page de connexion.
                            if (!data.success && data.error == "no_connected") {
                                openOverlayRedirectConnexion();
                            } else {
                                if (data.success && data.project) {
                                    // met a jour l'id du projet & incrémente de 1 le nombre de projets au quel l'utilisateur a participé.
                                    currentProject.id_projet_gnn = data.project.id;
                                    var projectsPublished = GeneanetLocalStorage.getItemWithDefaultValue("projectsPublished", [], "configuration");
                                    if (projectsPublished.indexOf(currentProject.id_projet_gnn) === -1) {
                                        projectsPublished.push(currentProject.id_projet_gnn);
                                    }
                                    GeneanetLocalStorage.setItem("projectsPublished", projectsPublished, "configuration");

                                    currentProject.proprietaire = data.project.sourcename_proprietaire ? data.project.sourcename_proprietaire : null;
                                    listProject[currentProject.id_projet_local] = currentProject;

                                    // on met à jour en BDD les infos sur le projet suite à sa synchronisation.
                                    var requestUpdateProjectSynchronized = GenerateSqlService.updateProjectSynchronized(
                                        currentProject.id_projet_local,
                                        currentProject.id_projet_gnn,
                                        currentProject.proprietaire
                                    );
                                    GeneanetDatabase.executeRequest(requestUpdateProjectSynchronized, function() {});
                                } else {
                                    console.error(data);
                                }

                                // on avance la progress bar et si elle est finie on lance la synchro des images.
                                GeneanetOverlay.advance();
                                if (GeneanetOverlay.progressBarIsFinished()) {
                                    GeneanetOverlay.close();
                                    synchroPicture();
                                }
                            }
                        })
                        .error(function() {
                            GeneanetOverlay.openOverlay(
                                $translate.instant("alert.fail_connect_gnn"),
                                $translate.instant("alert.fail_connect_gnn_more")
                            );
                        });
                })(currentProject);
            }
        }

        /**
         * Permet de changer l'état d'une synchronisation.
         * 
         * @param {integer} idProject Id du projet visé.
         * @param {integer} finish    1 si fini, 0 si non.
         * 
         * @return {integer} Retourne l'id de synchro.
         */
        var changeStepSynchro = function(idProject, finish) {
            var idSynchro = false;

            var params = {
                id: idProject,
                finish: finish,
                id_application: ConfigurationService.get("id_application")
            };

            // envoye à GNN.
            $.ajax({
                type: "POST",
                async: false,
                data: params,
                url: urlsWebservice.galleryUrl + "synchro/project",
                success: function(data) {
                    if (!data.success && data.error == "no_connected") {
                        openOverlayRedirectConnexion();
                    } else if (data.success && data.id_synchro_project) {
                        idSynchro = data.id_synchro_project;
                    }
                },
                error: function() {
                    GeneanetOverlay.openOverlay(
                        $translate.instant("alert.fail_connect_gnn"),
                        $translate.instant("alert.fail_connect_gnn_more")
                    );
                }
            });

            return idSynchro;
        }

        /**
         * Permet de démarrer une synchronisation sur un projet.
         * 
         * @param  {integer} idProject id du projet visé.
         * 
         * @return {integer} Retourne l'id de synchro.
         */
        var startSynchro = function(idProject) {
            return changeStepSynchro(idProject, 0);
        }

        /**
         * Permet de stopper une synchronisation sur un projet.
         * 
         * @param  {integer} idProject id du projet visé.
         * 
         * @return {integer} Retourne l'id de synchro.
         */
        var endSynchro = function(idProject) {
            return changeStepSynchro(idProject, 1);
        }

        /**
         * Permet de démarrer la synchronisation des images.
         * 
         * @param  {integer} nextIdProject Permet de savoir que c'est le premier projet que l'on synchronise et où l'on en est.
         */
        var synchroPicture = function(nextIdProject) {
            // on ouvre une progress bar lors du premier appel.
            if (typeof nextIdProject == "undefined") {
                nextIdProject = 0;
                var total = 0;
                for(var i in idsProjectsChecked) {
                    total += listProject[idsProjectsChecked[i]].count_picture;
                }

                managerSynchProgress.init(total);

                var divProgressBarFile = angular.element("<div>")
                    .attr("id", "progress-bar-file")
                    .html("<div></div>");
                var divDescription = angular.element("<div>")
                    .text($translate.instant("progress_bar.synchro_picture"));

                var buttonStopSynchro = angular.element("<button>")
                    .addClass("btn btn-gnn-secondary")
                    .click(function() {
                        angular.element(this).text($translate.instant("progress_bar.synchro_stop_progress"))
                            .attr("disabled", "disabled");
                        synchroInProgress = false;
                    })
                    .text($translate.instant("progress_bar.synchro_stop"));
                var divButton = angular.element("<div>")
                    .addClass("text-center")
                    .append(buttonStopSynchro);

                var htmlOverlay = angular.element("<div>")
                    .append(divProgressBarFile)
                    .append(divDescription)
                    .append(divButton);
                GeneanetOverlay.openProgressBar(htmlOverlay, 0, total);
            }

            // si on est pas au dernier projet et qu'on a une syncho en cours.
            if (nextIdProject < idsProjectsChecked.length && synchroInProgress) {
                var currentProject = listProject[idsProjectsChecked[nextIdProject]];

                // on va récupérer les 2 première photos à synchronisées.
                var requestGetPictures = GenerateSqlService.getPicturesNoSynchroByProject(currentProject.id_projet_local, 0, 2);
                GeneanetDatabase.executeRequest(requestGetPictures, function(tx, results) {
                    if (results.rows.length > 0) {
                        /**
                         * Permet de passer au fichier suivant à synchroniser.
                         */
                        var nextFile = function() {
                            // on avance la progress bar & on met à jour le nombre de photos synchronisées par l'utilisateur.
                            GeneanetOverlay.advance();
                            managerSynchProgress.advance();
                            ConfigurationService.edit(
                                "picturesPublished",
                                parseInt(ConfigurationService.get("picturesPublished"))+1,
                                function(){}
                            );

                            // si pas d'autre photo sur ce projet, on passe au suivant.
                            if (results.rows.length == 1) {
                                endSynchro(currentProject.id_projet_gnn);
                                synchroPicture(++nextIdProject);
                            } else {
                                // continue la synchro de ce projet.
                                synchroPicture(nextIdProject);
                            }
                        }

                        // start synchro
                        var idSynchro = startSynchro(currentProject.id_projet_gnn);
                        if (idSynchro === false) {
                            GeneanetOverlay.openOverlay(
                                $translate.instant("alert.fail_synchro"),
                                $translate.instant("alert.fail_synchro_more")
                            );
                            synchroInProgress = false;
                            return;
                        }

                        var ft = new FileTransfer();
                        if (angular.element("#progress-bar-file")) {
                            angular.element("#progress-bar-file > div").css("width", "0%");
                        }
                        ft.onprogress = function(progressEvent) {
                            if (progressEvent.lengthComputable) {
                                if (angular.element("#progress-bar-file")) {
                                    var pourcent = progressEvent.loaded / progressEvent.total * 100;
                                    angular.element("#progress-bar-file > div").css("width", pourcent+"%");
                                }
                            }
                        };

                        var currentFile = results.rows.item(0);
                        var fileUrl = currentFile.native_url;

                        // les options pour l'envoye du fichier.
                        var options = new FileUploadOptions();
                        options.fileKey = "file";
                        options.mimeType = "image/jpeg";
                        options.fileName = fileUrl.substr(fileUrl.lastIndexOf('/')+1);
                        options.params = {
                            id_synchro_project: idSynchro,
                            id_depot: currentFile.id_depot,
                            id_photo: currentFile.id_picture
                        };

                        // en cas d'échec d'envoie du fichier.
                        var failUpload = function(error) {
                            if (error.code === FileTransferError.FILE_NOT_FOUND_ERR) {
                                // fichier non trouvé, on delete de la BDD
                                var requestDelete = GenerateSqlService.deletePhotosFromIds(currentFile.id_picture);
                                GeneanetDatabase.executeRequest(requestDelete, function() {
                                    nextFile();
                                });
                            } else {
                                // gnn HS ou autre soucis
                                $scope.$on("GeneanetOverlayClose", function() {
                                    $route.reload();
                                });
                                GeneanetOverlay.openOverlay(
                                    $translate.instant("alert.fail_connect_gnn"),
                                    $translate.instant("alert.fail_connect_gnn_more")
                                );
                            }
                        }

                        // en cas de réussite d'envoie du fichier.
                        var successUpload = function(response) {
                            var data = JSON.parse(response.response);
                            if (!data.success && data.error == "no_connected") {
                                openOverlayRedirectConnexion();
                            } else if (data.success) {
                                // on supprime le fichier & la ligne en BDD si demandé au début de la synchro.
                                if (deleteFiles) {
                                    var eventNameDeletePicture = "deletePictureFile"+currentFile.id_picture;
                                    $scope.$on(eventNameDeletePicture, function(event, data) {
                                        if (data.success) {
                                            var requestDelete = GenerateSqlService.deletePhotosFromIds(currentFile.id_picture);
                                            GeneanetDatabase.executeRequest(requestDelete, function() {
                                                nextFile();
                                            });
                                        }
                                    });

                                    GeneanetFile.deleteFileFromUrl(currentFile.native_url, eventNameDeletePicture);
                                } else {
                                    // on passe la photo en état de synchronisée.
                                    var requestUpdateIsSync = GenerateSqlService.updatePictureIsSync(currentFile.id_picture);
                                    GeneanetDatabase.executeRequest(requestUpdateIsSync, function() {
                                        nextFile();
                                    });
                                }   
                            } else {
                                // pas réussi mais pas à cause de non connection, on passe au suivant et on notifiera l'utilisateur à la fin.
                                nextFile();
                                errorDuringUpload = true;
                            }
                        }
                        // lance l'upload du fichier.
                        ft.upload(fileUrl, urlGnn, successUpload, failUpload, options);
                    }
                });
            } else if(!synchroInProgress) {
                managerSynchProgress.stop();
            }
        }

        $scope.back = function () {
            window.history.back();
        }

        /**
         * Permet de rediriger si non connecté.
         */
        var redirectNoConnexion = function() {
            ConnexionService.setRedirect("/synchro/");
            $timeout(function() {
                $scope.$apply(function() {
                    $location.path("/connexion/").replace();
                });
            }, 11);
        }

        /**
         * Permet d'ouvrir l'overlay pour signaler que l'on doit avoir internet.
         * @return {[type]} [description]
         */
        var openOverlayRedirectConnexion = function() {
            $scope.$on("GeneanetOverlayClose", function() {
                redirectNoConnexion();
            });
            GeneanetOverlay.openOverlay(
                $translate.instant("alert.lost_connexion"),
                $translate.instant("alert.lost_connexion_more")
            );
        }

        /**
         * Event sur le chargement de la configuration pour générer un id unique pour l'application.
         */
        $scope.$on("configurationLoaded", function() {
            var uniqueId = (Math.random().toString(16)+"000000000").substr(2,8) +"-"+ new Date().getTime();
            ConfigurationService.add("id_application", uniqueId, function() {});
            checkConnexion();
        });
        ConfigurationService.loadConfig();

        /**
         * Event se déclenchant à la fin de la synchro des images.
         */
        $scope.$on("synchroPicturesFinished", function() {
            // on supprime les dépots vides pour faire un peu de ménage.
            var requestDeleteEmptyDepot = GenerateSqlService.deleteEmptyDepot();
            GeneanetDatabase.executeRequest(requestDeleteEmptyDepot, function() {
                // Lorsque l'utilisateur ferme l'overlay de validation de synchro, on actualise sur la page.
                $scope.$on("GeneanetOverlayClose", function() {
                    $location.path("/synchro/true").replace();
                    $route.reload();
                });
                // Si une erreur non bloquante est survenue durant la synchro on notifie l'utilisateur.
                if (errorDuringUpload) {
                    GeneanetOverlay.openOverlay(
                        $translate.instant("alert.error_during_upload"),
                        $translate.instant("alert.error_during_upload_more")
                    );
                } else {
                    // si non, on indique juste que la synchro est finie.
                    GeneanetOverlay.openOverlay(
                        $translate.instant("alert.synchro_finished"),
                        $translate.instant("alert.synchro_finished_more")
                    );
                }
            });
        });

        /**
         * Event se déclenchant lorsque toutes les vérifications pré-synchro ont été validées.
         */
        $scope.$on("checkFinished", function() {
            var listProjectTpl = new Array();
            // récupère les projets nécessitant d'être connecté (eventuellement filtré par le projet en cours si on vient de la page d'un projet).
            var requestProjectsSynchronize = GenerateSqlService.getProjectsNeedSynchronize(ProjectService.getCurrentIdProject());
            GeneanetDatabase.executeRequest(requestProjectsSynchronize, function(tx, results){
                for (var i = 0; i < results.rows.length; i++) {
                    var currentResult = results.rows.item(i);
                    var i18nIdPicture;

                    if (currentResult.count_picture <= 1) {
                        i18nIdPicture = 'list_projectpage.picture';
                    } else {
                        i18nIdPicture = 'list_projectpage.pictures';
                    }
                    currentResult.count_picture_msg = currentResult.count_picture + ' ' + $translate.instant(i18nIdPicture);
                    var tmpProject = {};
                    for (var j in currentResult) {
                        tmpProject[j] = currentResult[j];
                    }
                    listProject[tmpProject.id_projet_local] = tmpProject;
                    listProjectTpl.push(tmpProject);
                };

                // on affiche la liste des projets.
                $scope.$apply(function() {
                    $scope.listProject = listProjectTpl;
                    $scope.noProjet = results.rows.length > 0 ? false : true;
                });
            });
        });
    }
]