'use strict';

/**
 * Service spécifique à l'application Sauvons nos tombes
 * Permet de faire les requêtes plus facilement.
 **/
var projectService = [
    "$rootScope",
    "GeneanetCamera",
    "GeneanetDevice",
    "GeneanetFile",
    "GeneanetDate",
    "GeneanetDatabase",
    "GenerateSqlService",
    "GeneanetGeolocation",
    function($rootScope, GeneanetCamera, GeneanetDevice, GeneanetFile, GeneanetDate, GeneanetDatabase, GenerateSqlService, GeneanetGeolocation) {
        var nameEventSuccessResolveTakePicture = "projectServiceSuccessResolveTakePicture";
        var nameEventSuccessMovePicture = "projectServiceSuccessMovePicture";
        var nameEventSuccessSavePicture = "projectServiceSuccessSavePicture";
        var nameEventFinishedTakePicture = "projectServiceFinishedTakePicture";
        var nameEventGetLocalisation = "getLocalisation";

        var nameEventAfterFinishedTakePicture = null;

        var currentIdProject = null;
        var currentIdDepot = null;
        var currentIdPicture = null;
        var currentPictureEntry = null;

        var listenerOnGetLocalisation = null;

        /**
         * Permet de prendre une photo, la deplacer au bon endroit et la stocker en BDD.
         *
         * @param  {string} nameEvent Nom de l'évent levé une fois toutes les actions finies.
         */
        this.takePicture = function(nameEvent) {
            if (currentIdProject == null) {
                console.log("No project selected.");
            } else {
                nameEventAfterFinishedTakePicture = nameEvent;
                GeneanetCamera.prendreUnePhoto(successTakePicture, function() {
                    console.log(arguments);
                    window.history.back();
                } , false);
            }
        }

        var initOnListeners = function() {
            // dès qu'on arrive a récupérer le fichier de l'image une fois la photo faite, on la déplace.
            $rootScope.$on(nameEventSuccessResolveTakePicture, movePicture);

            // on va enregistrer en base de données la photo
            $rootScope.$on(nameEventSuccessMovePicture, savePicture);

            // on enregistre la fonction qui prévient que la prise de vue est finie.
            $rootScope.$on(nameEventFinishedTakePicture, function(event, argsFinishedTakePicture){
                $rootScope.$broadcast(nameEventAfterFinishedTakePicture, {idPicture: argsFinishedTakePicture.idPicture})
            });
        }

        var successTakePicture = function(pictureUrl) {
            // on va lever un event une fois l'image récupérer sous forme d'objet ainsi que le dossier gallery.
            window.resolveLocalFileSystemURL(pictureUrl, successResolvePicture, failResolvePicture);
        }

        var successResolvePicture = function(fileEntry) {
            // Récupère le fichier (path, etc).
            currentPictureEntry = fileEntry;

            // On va récupérer le path vers lequel on veut déplacer le fichier.
            if (GeneanetDevice.isAndroidPlatform()) {
                // Dans le cas d'android, on demande un accès au système de fichier pour aller
                // stocker le fichier dans Pictures/NotreDossier
                window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fs) {
                    fs.root.getDirectory(
                        'Pictures/'+GeneanetDevice.getNameDirectory(),
                        {create: true},
                        function (dirEntry) {
                            // Maintenant que l'on a le dossier de destination, on le transmet.
                            $rootScope.$broadcast(
                                nameEventSuccessResolveTakePicture,
                                {
                                    pictureFileEntry: currentPictureEntry,
                                    entityDestination: dirEntry
                                }
                            );
                        }, function () {
                            console.error('Erreur dans la récupération du dossier de destination');
                            console.log(arguments);
                        }
                    );
                }, function () {
                    console.error('Erreur lors de la récupération du système de fichier.');
                    console.log(arguments);
                });
            } else {
                GeneanetFile.getFileFromUrl(cordova.file.syncedDataDirectory, function(entityDestination) {
                    $rootScope.$broadcast(
                        nameEventSuccessResolveTakePicture,
                        {
                            pictureFileEntry: currentPictureEntry,
                            entityDestination: entityDestination
                        }
                    );
                });
            }
        }

        var failResolvePicture = function(codeError) {
            console.log("Error to resolve URI picture, code '"+codeError.code+"' ");
        }

        var movePicture = function(event, args) {
            var currentFile = args.pictureFileEntry;
            var entityDestination = args.entityDestination;
            var nameFileDestination = generateNameImage(GeneanetFile.getExtensionFile(currentFile.name));

            currentFile.moveTo(entityDestination, nameFileDestination, successMovePicture, failMovePicture)
        };

        var successMovePicture = function(fileEntry) {
            $rootScope.$broadcast(nameEventSuccessMovePicture, {pictureFileEntry: fileEntry});
        }

        var failMovePicture = function(codeError) {
            console.log("Error to move file, error code : '" + codeError +"'");
        }

        var savePicture = function(event, args) {
            var fileEntry = args.pictureFileEntry;
            var nativeUrl = fileEntry.nativeURL;

            if (currentIdDepot == null) {
                // on regarde si le projet est privé et que c'est sa première photo pour changer la localisation du projet.
                var requestGetProject = GenerateSqlService.getProjectsWithCountPictures(currentIdProject);
                GeneanetDatabase.executeRequest(
                    requestGetProject,
                    function(tx, results) {
                        if (results.rows.length !== 1) {
                            return;
                        }
                        var projectInfos = results.rows.item(0);
                        // si le projet appartient à l'utilisateur.
                        if (
                            (projectInfos.proprietaire !== null || projectInfos.id_projet_gnn === null) &&
                            projectInfos.count_picture <= 1
                        ) {
                            // on desactive l'éventuel précédent listener placé sur la localisation.
                            if (listenerOnGetLocalisation != null) {
                                listenerOnGetLocalisation();
                                listenerOnGetLocalisation = null;
                            }

                            // on greffe un nouveau listener sur la localisation.
                            listenerOnGetLocalisation = $rootScope.$on(
                                "updateProject",
                                (function(idProject) {
                                    return function(event, data) {
                                        if (data.success) {
                                            var requestUpdateLocalisationProject = GenerateSqlService.updateProjectLocalisation(
                                                idProject,
                                                data.position.coords.latitude,
                                                data.position.coords.longitude
                                            );
                                            GeneanetDatabase.executeRequest(
                                                requestUpdateLocalisationProject,
                                                function(tx, results) {
                                                }
                                            );
                                        }
                                    }
                                })(currentIdProject)
                            );

                            // on essaye de localiser.
                            GeneanetGeolocation.getCurrentPosition("updateProject", 60000, 5000, true);
                        }
                    }
                );
                var reqInsertDepot = GenerateSqlService.insertionDepot(currentIdProject, null);
                GeneanetDatabase.executeRequest(
                    reqInsertDepot,
                    function(tx, results) {
                        currentIdDepot = results.insertId;
                        $rootScope.$broadcast(nameEventSuccessMovePicture, {pictureFileEntry: fileEntry});
                    }
                );
            } else {
                var reqInsertPicture = GenerateSqlService.insertionPhoto(nativeUrl, currentIdDepot, false);
                GeneanetDatabase.executeRequest(
                    reqInsertPicture,
                    function(tx, results) {
                        var idPictureInsert = results.insertId;
                        currentIdPicture = idPictureInsert;
                        $rootScope.$broadcast(nameEventFinishedTakePicture, {idPicture: idPictureInsert});
                    }
                );
            }
        }

        /**
         * Permet de générer un nom d'image depuis la date courrante.
         *
         * @param {string} extension Nom de l'extention de fichier.
         *
         * @return {string} Renvoie le nom générer.
         */
        var generateNameImage = function(extension) {
            var elementsToGenerateName = new Array();
            elementsToGenerateName.push("GNN");

            var currentDate = new Date();
            var year = String(currentDate.getFullYear());
            var month = String(GeneanetDate.formatedTime(currentDate.getMonth()+1));
            var day = String(GeneanetDate.formatedTime(currentDate.getDate()));
            var hour = String(GeneanetDate.formatedTime(currentDate.getHours()));
            var minute = String(GeneanetDate.formatedTime(currentDate.getMinutes()));
            var second = String(GeneanetDate.formatedTime(currentDate.getSeconds()));

            elementsToGenerateName.push(year+month+day);
            elementsToGenerateName.push(hour+minute+second);

            return elementsToGenerateName.join("_") + "." + extension;
        }

        this.setCurrentIdProject = function(newCurrentIdProject) {
            currentIdProject = newCurrentIdProject;
        }

        this.getCurrentIdProject = function() {
            return currentIdProject;
        }

        this.setCurrentIdDepot = function(newCurrentIdDepot) {
            currentIdDepot = newCurrentIdDepot;
        }

        this.getCurrentIdDepot = function() {
            return currentIdDepot;
        }

        this.setCurrentIdPicture = function(newCurrentIdPicture) {
            currentIdPicture = newCurrentIdPicture;
        }

        this.getCurrentIdPicture = function() {
            return currentIdPicture;
        }

        initOnListeners();
    }
]