'use strict';

/**
 * Permet de prendre une photo.
 * Méthodes Utilisable :
 *     ** prendreUnePhoto **
 */
angular.module('GeneanetService').provider( 'GeneanetCamera',[
    function () {
        /*  Qualité de la camera    */
        var qualityCamera = 50;

        /**
        *   Permet de définir la qualité par default de la camera
        **/
        this.defaultQuality = function ( proposedQuality){
            if(proposedQuality>=0 && proposedQuality<=100)
                qualityCamera = proposedQuality;
            else
                qualityCamera = 50;
        }

        /**
            Instanciation de GeneanetCamera
        **/
        this.$get = [function(){
            var GeneanetCamera = new GeneanetCameraPrototype();

            GeneanetCamera.setQualityCamera(qualityCamera);

            return GeneanetCamera;
        }];
    }
]);

/**
 * Contient toutes les méthodes utiles pour manipuler l'appareil photo du périphérique.
 */
var GeneanetCameraPrototype = function () {
    var qualityCamera;
    var destinationTypeCamera;

    this.prendreUnePhoto = function(successCamera, failCamera, flagSaveToPhotoAlbum) {
        if (navigator.camera) {

            var currentQualityCamera;
            if (qualityCamera) {
                currentQualityCamera = qualityCamera;
            } else {
                currentQualityCamera = 50;
            }

            var currentDestinationTypeCamera;
            if (destinationTypeCamera) {
                currentDestinationTypeCamera = destinationTypeCamera;
            } else {
                currentDestinationTypeCamera = Camera.DestinationType.FILE_URI;
            }

            navigator.camera.getPicture(
                successCamera,
                failCamera,
                {
                    quality: currentQualityCamera,
                    destinationType: currentDestinationTypeCamera,
                    saveToPhotoAlbum: flagSaveToPhotoAlbum,
                    correctOrientation: true
                }
            );
        }
    }

    /***    Méthode SET     ***/

    this.setQualityCamera = function ( newQualityCamera) {
        qualityCamera = newQualityCamera;
    };

    this.setDestinationTypeCamera = function ( newDestinationTypeCamera) {
        destinationTypeCamera = newDestinationTypeCamera;
    };


    /***    Méthode GET ***/

    this.getQualityCamera = function () {
        return qualityCamera;
    };

    this.getDestinationTypeCamera = function () {
        return destinationTypeCamera;
    };
};