'use strict';

/**
 *   Permet de creer une Base de Données.
 *   Méthodes Utilisable :
 *       ** initDataBase **
 *       ** chooseConstante **
 *       ** initConnexion **
 *       ** changeVersion **
 */
angular.module('GeneanetService').provider('GeneanetDatabase',[
    function (){
        var nameDatabase = "";
        var versionDataBase = "";
        var displayName = "";
        var tailledatabase = 0;
        var tabRequest = new Array();

        /**
        *Ouverture de la base de données
        **/
        this.initDataBase =  function ( name, version, displayname, size, newTabRequest){
            nameDatabase = name;
            versionDataBase = version;
            displayName = displayname;
            tailledatabase = size;
            tabRequest = newTabRequest;
        };


        this.$get = [
            function (){
                var GeneanetDatabase = new GeneanetDatabasePrototype();

                GeneanetDatabase.setNameDatabase(nameDatabase);
                GeneanetDatabase.setVersionDataBase(versionDataBase);
                GeneanetDatabase.setDisplayName(displayName);
                GeneanetDatabase.setTailledatabase(tailledatabase);
                
                var tabRequestClean = new Array();
                for (var i in tabRequest) {
                    var request = GeneanetDatabase.createRequest(tabRequest[i]);
                    request.setIsReadAndWrite(true);
                    tabRequestClean.push(request);
                }
                GeneanetDatabase.setTabRequest(tabRequestClean);

                return GeneanetDatabase;
            }
        ];
    }
]);


/**
 * Contient toutes les méthodes utiles pour créer une base de donnée et faire des requètes
 */
var GeneanetDatabasePrototype = function () { 
    var database;
    var nameDatabase;
    var versionDataBase;
    var displayName;
    var tailledatabase;
    var tabRequest;

    /**
     * Ouverture de la base de données
     */
    this.initConnexion = function() {
        database = window.openDatabase(nameDatabase, versionDataBase, displayName, tailledatabase);
        for(var i in tabRequest){
            this.executeRequest(tabRequest[i], function(){});
        }
        return database;
    };

    var errorDataBase = function(errorDb) {
        console.log("Erreur SQL: ");
        console.log(errorDb);
    }

    /**
     * Permet de changer la version de la base de donnée
     * Il est préférable de changer la version à chaque modification Importante
     * Ou a chaque version de votre Web App
     */
    this.changeVersion = function(newVersionDataBase) {
        database.changeVersion(this.getVersionDataBase(), newVersionDataBase);  
    }

    this.executeRequest = function(requestObject, successRequest) {
        if (requestObject.constructor != requestPrototype) {
            console.error("Error to execute Request SQL. Need object requestPrototype.");
        } else {
            var execSql = function(tx){
                tx.executeSql(
                    requestObject.getRequest(),
                    requestObject.getParams(),
                    successRequest,
                    errorDataBase
                );
            }

            if (requestObject.isOnlyRead()) {
                database.readTransaction(
                    execSql,
                    errorDataBase,
                    null
                );
            } else {
                database.transaction(
                    execSql,
                    errorDataBase,
                    null
                );
            }
        }
    }

    /**
     * Object to manage request.
     */
    var requestPrototype = function() {
        /**
         * SQL request.
         * 
         * @type {String}
         */
        var request = null;

        /**
         * Optionnal params to prepared request.
         * 
         * @type {Array}
         */
        var params = null;

        /**
         * Flag to indicate if request need only read or not.
         * 
         * @type {Boolean}
         */
        var onlyRead = true;

        /**
         * Set new sql request.
         * 
         * @param {string} newRequest New sql request.
         */
        this.setRequest = function(newRequest) {
            request = newRequest;
        }

        /**
         * Get sql request to execute.
         * 
         * @return {String} Return sql request to execute.
         */
        this.getRequest = function() {
            return request;
        }

        /**
         * Set new params.
         * 
         * @param {Array} newParams Array contains new params to request.
         */
        this.setParams = function(newParams) {
            params = newParams;
        }

        /**
         * Get request's params.
         * 
         * @return {Array} Return request's params.
         */
        this.getParams = function() {
            return params;
        }

        /**
         * Set this request need only access to read.
         */
        this.setIsOnlyRead = function() {
            onlyRead = true;
        }

        /**
         * Set this request need access to read and to write.
         */
        this.setIsReadAndWrite = function() {
            onlyRead = false;
        }

        /**
         * Indicate if request need only read or not.
         * 
         * @return {Boolean} True if only read, false also.
         */
        this.isOnlyRead = function() {
            if (onlyRead) {
                return true;
            } else {
                return false;
            }
        }
    }

    this.createRequest = function(request, params, onlyRead) {
        var requestObject = new requestPrototype();
        requestObject.setRequest(request);

        if (typeof params == "undefined") {
            params = new Array();
        }
        requestObject.setParams(params);
        if (typeof onlyRead != "undefined" && !onlyRead) {
            requestObject.setIsReadAndWrite();
        } else {
            requestObject.setIsOnlyRead();
        }
        return requestObject;
    }

    this.generateClauseWhereIn = function(values) {
        if (typeof values != "object") {
            var tmp = new Array();
            tmp.push(values);
            values = tmp;
        }

        return "('"+values.join("','")+"')";
    }

    /***    Méthode SET ***/
    this.setDatabase = function ( newDatabase){
        database = newDatabase;
    }

    this.setNameDatabase = function ( newNameDatabase){
        nameDatabase = newNameDatabase;
    }

    this.setDisplayName = function ( newDisplayName){
        displayName = newDisplayName;
    }

    this.setVersionDataBase = function ( newVersionDataBase){
        versionDataBase = newVersionDataBase;
    }

    this.setTailledatabase = function ( newTailledatabase){
        tailledatabase = newTailledatabase;
    }

    this.setTabRequest = function ( newTabRequest){
        tabRequest = newTabRequest;
    }

    /***    Méthode GET ***/
    this.getDatabase = function (){
        return database;
    }

    this.getNameDatabase = function (){
        return nameDatabase;
    }

    this.getVersionDataBase = function (){
        return versionDataBase;
    }

    this.getDisplayName = function (){
        return displayName;
    }

    this.getTailledatabase = function (){
        return tailledatabase;
    }

    this.getTabRequest = function (){
        return tabRequest;
    }
};