'use strict';

/**
 * On définie ici les fonctions de plugin commun
 * On définie aussi des variables spécifique à l'application
 * à plusieurs controleurs.
 *
 * Mémo pour les paths : https://github.com/apache/cordova-plugin-file/blob/master/doc/index.md#where-to-store-files
 */
angular.module('GeneanetService').provider('GeneanetDevice', [
    function () {
        /**
         * @var {string} Default name directory to application.
         */
        var nameDirectory = null;

        this.setNameDirectory = function(newNameDirectory) {
            nameDirectory = newNameDirectory;
        }

        this.$get = [
            'GeneanetApp',
            '$route',
            function(GeneanetApp, $route){
                var GeneanetDevice = new GeneanetDevicePrototype($route);

                if (nameDirectory == null || nameDirectory.trim().length == 0 ) {
                    nameDirectory = GeneanetApp.getName();
                }
                GeneanetDevice.setNameDirectory(nameDirectory);

                return GeneanetDevice;
            }
        ];
    }
]);

var GeneanetDevicePrototype = function($route) {
    var thisCurrentInstance = this;

    var PLATFORM_ANDROID = "android";
    var PLATFORM_IOS = "ios";

    var $route = $route;

    /**
     * @var {string} Default name directory to application.
     */
    var nameDirectory = null;

    //Permet de vérouiller la position du téléphone si l'on est sous Android
    this.determineOrientationCurrentPage = function() {
        if (thisCurrentInstance.isAndroidPlatform()){
            if ($route.current.$$route.orientationLock) {
                window.plugins.orientationLock.lock($route.current.$$route.orientationLock);
            } else {
                window.plugins.orientationLock.unlock();
            }
        }
    }

    this.getPathApplication = function() {
        if (thisCurrentInstance.isAndroidPlatform() || thisCurrentInstance.isIosPlatform()) {
            return cordova.file.applicationDirectory;
        } else {
            return nameDirectory;
        }
    }

    this.getPathExternalApplication = function() {
        if (thisCurrentInstance.isAndroidPlatform() || thisCurrentInstance.isIosPlatform()) {
            return cordova.file.externalApplicationStorageDirectory;
        } else {
            return nameDirectory;
        }
    }
    
    this.getPathData = function() {
        if (thisCurrentInstance.isAndroidPlatform() || thisCurrentInstance.isIosPlatform()) {
            return cordova.file.dataDirectory;
        } else {
            return nameDirectory;
        }
    }

    /**
     * Permet de savoir si on est sur un appareil android ou non.
     * 
     * @return {Boolean} True => platform android, false also.
     */
    this.isAndroidPlatform = function() {
        if (typeof device != 'undefined' && PLATFORM_ANDROID == device.platform.toLowerCase()){
            return true;
        } else {
            return false;
        }
    }

    /**
     * Permet de savoir si on est sur un appareil ios ou non.
     * 
     * @return {Boolean} True => platform ios, false also.
     */
    this.isIosPlatform = function() {
        if (typeof device != 'undefined' && PLATFORM_IOS == device.platform.toLowerCase()){
            return true;
        } else {
            return false;
        }
    }

    this.setNameDirectory = function(newNameDirectory) {
        nameDirectory = newNameDirectory;
    }

    this.getNameDirectory = function() {
        return nameDirectory;
    }

    this.accessInternetStarted = function() {
        if (navigator.connection && navigator.connection.type !== Connection.NONE) {
            return true;
        } else {
            return false;
        }
    }

    this.wifiStarted = function() {
        if (navigator.connection && navigator.connection.type === Connection.WIFI) {
            return true;
        } else {
            return false;
        }
    }
}
