'use strict';

/**
 * Service qui s'occupe de la localisation
 */
angular.module('GeneanetService').service('GeneanetGeolocation',[
    "$rootScope",
    function ($rootScope) {
        /**
         * Permet de connaitre la position actuelle de l'utilisateur.
         * 
         * @param  {string}  nameEvent          Nom de l'évènement sur lequel broadcast le résultat.
         * @param  {integer} timeCache          Temps maximum en milliseconde de valider du cache pour la position.
         * @param  {integer} timeout            Temps maximum en milliseconde pour tenter de déterminer la position.
         * @param  {boolean} enableHighAccuracy A true : permet d'exploiter la meilleur méthode pour
         *                                      localiser, false seulement via réseau.
         */
        this.getCurrentPosition = function(nameEvent, timeCache, timeout, enableHighAccuracy) {
            var options = {
                maximumAge: timeCache ? timeCache : null,
                timeout: timeout ? timeout : null,
                enableHighAccuracy: enableHighAccuracy ? enableHighAccuracy : false
            };

            var fnSuccess = function(position) {
                $rootScope.$broadcast(nameEvent, {success: true, position: position});
            }

            var fnFail = function(positionError) {
                $rootScope.$broadcast(nameEvent, {success: false, positionError: positionError});
            }
            navigator.geolocation.getCurrentPosition(fnSuccess, fnFail, options);
        }
    }
]);